/*!	\file		Cicada.c
	\author		Garth Santor
	\date		2019-02-12
	\version	1.0.0

	Cycada Cycles solution in C.
*/

#include <stdio.h>
#include <stdlib.h>

#define MAX_CICADA_CYCLES 20


/*!	Count the number of years that the cicada will coincide with predators.

	\param kCicadaCycle [in] the number of years in the cicada's life cycle.
	\param kTrialYears [in] the number of years to test against the cicada's cycle.
	\return the number of predators that align with the cicada's life cycle.
*/
int num_predators(const int kCicadaCycle, const int kTrialYears) {
	int predatorCount = 0;
	for (int year = 1; year <= kTrialYears; ++year) {
		if (year%kCicadaCycle == 0) {
			predatorCount += ((year % 2) == 0);
			predatorCount += ((year % 3) == 0);
			predatorCount += ((year % 4) == 0);
			predatorCount += ((year % 6) == 0);
		}
	}
	return predatorCount;
}



/*!	Cicada trial result.
*/
struct CicadaTrialResult {
	int cycleYears;
	int numberPredatorYears;
};

/*!	qsort compatible comparison callback function.
	\param pvA [in] pointer to the first argument.
	\param pvB [in] pointer to the second argument.
	\return -ve if a<b, 0 if a==b, +ve if a>b
*/
int compare_predators(const void * pvA, const void * pvB) {
	const struct CicadaTrialResult *pA = pvA, *pB = pvB;
	return pA->numberPredatorYears - pB->numberPredatorYears;
}


/*!	Process entry point - run the cicada trials.
*/
int main() {
	// Query the user for the number of trials to perform.
	printf("Enter the number of years to test: ");
	int trialYears;
	if (scanf_s("%d", &trialYears) != 1) {
		fprintf(stderr, "Error: failed to get a valid number.\n");
		return EXIT_FAILURE;
	}
	if (trialYears <= 0) {
		fprintf(stderr, "Error: the number of trial years must be a natural number.\n");
		return EXIT_FAILURE;
	}


	// Make an array of results.
	struct CicadaTrialResult results[MAX_CICADA_CYCLES];
	for (int cicadaCycle = 1; cicadaCycle <= MAX_CICADA_CYCLES; ++cicadaCycle) {
		results[cicadaCycle - 1].cycleYears = cicadaCycle;
		results[cicadaCycle - 1].numberPredatorYears = num_predators(cicadaCycle, trialYears);
	}


	// Output a table of trial results.
	puts("cicada cycle\t#predators");
	for (int cicadaCycle = 1; cicadaCycle <= MAX_CICADA_CYCLES; ++cicadaCycle)
		printf("%d\t\t%d\n", results[cicadaCycle - 1].cycleYears, results[cicadaCycle - 1].numberPredatorYears);


	// Output a table of the best 4 results
	qsort(results, MAX_CICADA_CYCLES, sizeof(struct CicadaTrialResult), compare_predators);
	puts("\n\n\nBest Cycles\n\ncicada cycle\t#predators");
	for (int cicadaCycle = 1; cicadaCycle <= 4; ++cicadaCycle)
		printf("%d\t\t%d\n", results[cicadaCycle - 1].cycleYears, results[cicadaCycle - 1].numberPredatorYears);


	return EXIT_SUCCESS;
}